/*   edit.c */

/*  Copyright (C)  Jean-Franois Wauthy under GNU GPL
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "edit_dialog.h"

/* Edition */
void treeview_activate_cb(GtkWidget *widget, GtkTreePath *path, GtkTreeViewColumn *col, gpointer data)
{
  xmlNodePtr node;
  GtkTreeIter iter;
  GValue val = { 0, };

  GtkWidget *dialog;
  GtkWidget *header;
  GtkWidget *header_image;
  gchar *header_text;

  GtkWidget* name_entry;
  GtkWidget* command_entry;
  GtkWidget *icon_entry;

  /* Retrieve the xmlNodePtr of the menu entry */
  gtk_tree_model_get_iter(GTK_TREE_MODEL(menueditor_app.treestore), &iter, path);
	
  gtk_tree_model_get_value (GTK_TREE_MODEL(menueditor_app.treestore), &iter, POINTER_COLUMN, &val);
  node = g_value_get_pointer(&val);

  if(!node)
    return;

  /* Create dialog for editing */
  dialog = gtk_dialog_new_with_buttons(_("Edit menu entry"),
				       GTK_WINDOW(menueditor_app.main_window),
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_CANCEL,
				       GTK_RESPONSE_CANCEL,
				       GTK_STOCK_OK,
				       GTK_RESPONSE_OK,NULL);


  /* Header */
#ifndef DISABLE_CVS
  header_image = gtk_image_new_from_stock("gtk-justify-fill", GTK_ICON_SIZE_LARGE_TOOLBAR);
  header_text = g_strdup_printf("%s", _("Edit menu entry"));
#warning only works with CVS version of libxfcegui4 (use --disable-cvs to not use CVS functions)
  header = create_header_with_image (header_image, header_text);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), header, FALSE, FALSE, 0);
  g_free (header_text);
#else
  header_text = g_strdup_printf("%s", _("Edit menu entry"));
  header = create_header (NULL, header_text);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), header, FALSE, FALSE, 0);
  g_free (header_text);
#endif

  /* Choose the edition dialog */
  if(!xmlStrcmp(node->name,(xmlChar*)"separator") ){
    gtk_widget_destroy (dialog);
    return;
  }else if(!xmlStrcmp(node->name,(xmlChar*)"app")){
    GtkWidget* name_label;
    GtkWidget* command_label;
    GtkWidget* table;
    GtkWidget *button_browse;
    GtkWidget *hbox_command;
    GtkWidget *icon_label;
    GtkWidget *button_browse2;
    GtkWidget *hbox_icon;

    table = gtk_table_new(3,2,FALSE);

    /* Icon */
    hbox_icon = gtk_hbox_new(FALSE, 0);
    icon_label = gtk_label_new(_("Icon :"));
    icon_entry = gtk_entry_new();
    button_browse2 = gtk_button_new_with_label("...");
    gtk_box_pack_start (GTK_BOX (hbox_icon), icon_entry, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_icon), button_browse2, FALSE, FALSE, 0);
    g_signal_connect ((gpointer) button_browse2, "clicked", G_CALLBACK (browse_command_cb), icon_entry);

    hbox_command = gtk_hbox_new(FALSE,0);
    name_label = gtk_label_new(_("Name :"));
    command_label = gtk_label_new(_("Command :"));

    name_entry = gtk_entry_new();
    command_entry = gtk_entry_new();

    button_browse = gtk_button_new_with_label("...");

    gtk_box_pack_start (GTK_BOX (hbox_command), command_entry, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_command), button_browse, FALSE, FALSE, 0);
    g_signal_connect ((gpointer) button_browse, "clicked", G_CALLBACK (browse_command_cb), command_entry);

    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    gtk_container_set_border_width(GTK_CONTAINER(table),10);

    gtk_table_attach(GTK_TABLE(table), name_label, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), command_label, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), hbox_command, 1, 2, 1, 2, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), icon_label, 0, 1, 2, 3, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), hbox_icon, 1, 2, 2, 3, GTK_FILL, GTK_SHRINK, 0, 0);

    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);

    gtk_entry_set_text(GTK_ENTRY(name_entry), xmlGetProp(node,"name"));
    gtk_entry_set_text(GTK_ENTRY(command_entry), xmlGetProp(node,"cmd"));
    if(xmlGetProp(node,"icon"))
      gtk_entry_set_text(GTK_ENTRY(icon_entry), xmlGetProp(node,"icon"));

    gtk_window_set_default_size(GTK_WINDOW(dialog),300,150);
  }else if(!xmlStrcmp(node->name,(xmlChar*)"menu")){
    GtkWidget* name_label;
    GtkWidget* table;

    name_label = gtk_label_new(_("Name :"));
    name_entry = gtk_entry_new();

    table = gtk_table_new(2,2,FALSE);

    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    gtk_container_set_border_width(GTK_CONTAINER(table),10);

    gtk_table_attach(GTK_TABLE(table), name_label, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);

    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);

    gtk_entry_set_text(GTK_ENTRY(name_entry), xmlGetProp(node,"name"));

    gtk_window_set_default_size(GTK_WINDOW(dialog),200,100);
  }else if(!xmlStrcmp(node->name,(xmlChar*)"builtin")){
    GtkWidget* name_label;
    GtkWidget* table;

    name_label = gtk_label_new(_("Name :"));
    name_entry = gtk_entry_new();

    table = gtk_table_new(2,2,FALSE);

    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    gtk_container_set_border_width(GTK_CONTAINER(table),10);

    gtk_table_attach(GTK_TABLE(table), name_label, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);

    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);

    gtk_entry_set_text(GTK_ENTRY(name_entry), xmlGetProp(node,"name"));

    gtk_window_set_default_size(GTK_WINDOW(dialog),200,100);
  }else if(!xmlStrcmp(node->name,(xmlChar*)"title")){
    GtkWidget* name_label;
    GtkWidget* table;

    name_label = gtk_label_new(_("Title :"));
    name_entry = gtk_entry_new();

    table = gtk_table_new(2,2,FALSE);

    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    gtk_container_set_border_width(GTK_CONTAINER(table),10);

    gtk_table_attach(GTK_TABLE(table), name_label, 0, 1, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);
    gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1, GTK_FILL, GTK_SHRINK, 0, 0);

    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);

    gtk_entry_set_text(GTK_ENTRY(name_entry), xmlGetProp(node,"name"));

    gtk_window_set_default_size(GTK_WINDOW(dialog),200,100);
  }

  gtk_widget_show_all(dialog);

  /* Commit change if needed */
  if(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK){
    menueditor_app.menu_modified=TRUE;
    gtk_widget_set_sensitive(menueditor_app.file_menu.save,TRUE);
    gtk_widget_set_sensitive(menueditor_app.main_toolbar.save,TRUE);

    if(!xmlStrcmp(node->name,(xmlChar*)"app")){
      GdkPixbuf *icon = NULL;
      GdkPixbuf *tmpicon = NULL;
      gchar *name=NULL;
      gchar *command=NULL;

      xmlSetProp(node,"name",gtk_entry_get_text(GTK_ENTRY(name_entry)));
      xmlSetProp(node,"cmd",gtk_entry_get_text(GTK_ENTRY(command_entry)));

      if(strlen(gtk_entry_get_text(GTK_ENTRY(icon_entry)))==0){
	xmlAttrPtr icon_prop;

	/* Remove the property in the xml tree */
	icon_prop = xmlHasProp(node, "icon");
	xmlRemoveProp(icon_prop);
      }else{
	tmpicon = gdk_pixbuf_new_from_file (gtk_entry_get_text(GTK_ENTRY(icon_entry)),
					    NULL);
	icon = gdk_pixbuf_scale_simple (tmpicon, icon_size, icon_size,
					GDK_INTERP_BILINEAR);
	g_object_unref (G_OBJECT (tmpicon));
	xmlSetProp(node,"icon",gtk_entry_get_text(GTK_ENTRY(icon_entry)));
      }

      name = g_strdup_printf(NAME_FORMAT,
			     gtk_entry_get_text(GTK_ENTRY(name_entry)));
      command = g_strdup_printf(COMMAND_FORMAT,
				gtk_entry_get_text(GTK_ENTRY(command_entry)));
				      
      gtk_tree_store_set (menueditor_app.treestore, &iter, 0, NULL, 
			  ICON_COLUMN, icon,
			  NAME_COLUMN, name,
			  COMMAND_COLUMN, command, -1);

      g_free(name);
      g_free(command);
    }else if(!xmlStrcmp(node->name,(xmlChar*)"menu")){
      gchar *name=NULL;
      
      name = g_strdup_printf(MENU_FORMAT,
			     gtk_entry_get_text(GTK_ENTRY(name_entry)));

      gtk_tree_store_set (menueditor_app.treestore, &iter, 0, NULL, 
			  NAME_COLUMN, name, -1);
      xmlSetProp(node,"name",gtk_entry_get_text(GTK_ENTRY(name_entry)));

      g_free(name);
    }else if(!xmlStrcmp(node->name,(xmlChar*)"builtin")){
      gchar *name=NULL;
      
      name = g_strdup_printf(BUILTIN_FORMAT,
			     gtk_entry_get_text(GTK_ENTRY(name_entry)));

      gtk_tree_store_set (menueditor_app.treestore, &iter, 0, NULL, 
			  NAME_COLUMN, name, -1);
      xmlSetProp(node,"name",gtk_entry_get_text(GTK_ENTRY(name_entry)));

      g_free(name);
    }else if(!xmlStrcmp(node->name,(xmlChar*)"title")){
      gchar *name=NULL;
      
      name = g_strdup_printf(TITLE_FORMAT,
			     gtk_entry_get_text(GTK_ENTRY(name_entry)));

      gtk_tree_store_set (menueditor_app.treestore, &iter, 0, NULL, 
			  NAME_COLUMN, name, -1);
      xmlSetProp(node,"name",gtk_entry_get_text(GTK_ENTRY(name_entry)));
      
      g_free(name);
    }
    
  }
  gtk_widget_destroy (dialog);
}
